# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.0.0

import enum

import winrt.system
from winui3._winui3_microsoft_ui_xaml_hosting import (
    DesktopWindowXamlSource,
    DesktopWindowXamlSourceGotFocusEventArgs,
    DesktopWindowXamlSourceTakeFocusRequestedEventArgs,
    ElementCompositionPreview,
    WindowsXamlManager,
    XamlShutdownCompletedOnThreadEventArgs,
    XamlSourceFocusNavigationRequest,
    XamlSourceFocusNavigationResult,
)
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from winui3._winui3_microsoft_ui_xaml_hosting import (
        DesktopWindowXamlSource_Static,
    )

__all__ = [
    "XamlSourceFocusNavigationReason",
    "DesktopWindowXamlSource",
    "DesktopWindowXamlSourceGotFocusEventArgs",
    "DesktopWindowXamlSourceTakeFocusRequestedEventArgs",
    "ElementCompositionPreview",
    "WindowsXamlManager",
    "XamlShutdownCompletedOnThreadEventArgs",
    "XamlSourceFocusNavigationRequest",
    "XamlSourceFocusNavigationResult",
]

class XamlSourceFocusNavigationReason(enum.IntEnum):
    PROGRAMMATIC = 0
    RESTORE = 1
    FIRST = 3
    LAST = 4
    LEFT = 7
    UP = 8
    RIGHT = 9
    DOWN = 10

